package nl.tue.declare.appl.framework.engine;

/**
 * <p>Title: DECLARE</p>
 *
 * <p>Description: </p>
 *
 * <p>Copyright: Copyright (c) 2006</p>
 *
 * <p>Company: TU/e</p>
 *
 * @author Maja Pesic
 * @version 1.0
 */
import java.io.*;
import java.util.*;

import org.processmining.models.operationalsupport.Response;
import org.processmining.models.operationalsupport.net.client.OperationalSupportClient;


import nl.tue.declare.appl.framework.recommendation.*;
import nl.tue.declare.datamanagement.*;
import nl.tue.declare.datamanagement.assignment.*;
import nl.tue.declare.domain.instance.*;
import nl.tue.declare.domain.model.*;
import nl.tue.declare.domain.organization.*;
import nl.tue.declare.execution.*;
import nl.tue.declare.execution.msg.*;
import nl.tue.declare.graph.model.*;
import nl.tue.declare.utils.*;
public class EngineAssignmentCoordinator{

  private String assignmentString;
  private Assignment assignment;
  private Execution execution;
  private AssignmentState assignmentState;

  private List<RemoteWorklist> worklists;

  private LTLInstanceExecutionHandler strategy;

  private RecommendationCoordinator recommendationCoordinator;

  private IAssignmentStateListener assignmentStateListener;

  private File file;
  
  /**
   *
   * @param assignment Assignment
   * @param file File
   */
  protected EngineAssignmentCoordinator(Assignment assignment, File file) throws
      Throwable {
    super();
    this.file = file;
    this.assignment = assignment;
    this.assignmentState = new AssignmentState(this.assignment);
    this.execution = new Execution(this.assignment);
       
    this.assignmentState.addWorkItemListener(this.execution);
    this.assignmentStateListener = null;

    this.worklists = Collections.synchronizedList(new ArrayList<RemoteWorklist>());

    this.recommendationCoordinator = null;
    // read the assignment from the file
    this.getAssignmentAndViewString(file.getAbsolutePath());

    // here we choose the strategy and the
    // algorithm for translation of LTL to automata
    // in this case, we use the general strategy and the algorithm for finite traces
    this.strategy = new LTLInstanceExecutionHandler(this.assignment);
    this.getAssignmentState().addListener(this.strategy);
  }

  public boolean sameModel(EngineAssignmentCoordinator c) {
    return c.file.equals(this.file);
  }

  public File getFile() {
    return file;
  }

  public void setAssignment(Assignment assignment) throws Throwable {
    this.assignment = assignment;
    this.assignmentState = new AssignmentState(this.assignment);
    this.execution = new Execution(this.assignment);
    this.assignmentState.addWorkItemListener(this.execution);

    this.strategy.reset(assignment);
    this.getAssignmentState().addListener( (LTLInstanceExecutionHandler)this.
                                          strategy);
  }

  public String toString() {
    return Integer.toString(assignment.getId());
  }

  private AssignmentState getAssignmentState() {
    return this.assignmentState;
  }

  public String getAssignmentString() {
    return this.assignmentString;
  }

  private synchronized void createState() {
    try {
      this.getAssignmentState().createState();
      if (this.assignmentStateListener != null) {
        this.assignmentStateListener.stateChanged(getAssignmentState());
      }
    }
    catch (Throwable ex) {
      ex.printStackTrace();
    }
  }

  public void setAssignmentStateListener(IAssignmentStateListener listener) {
    this.assignmentStateListener = listener;
    this.createState();
  }

  /**
   *
   * @param worklist RemoteWorklist
   */
  public void removeWorkList(RemoteWorklist worklist) {
    if (worklist != null) {
      synchronized (worklists) {
        worklists.remove(worklist);
      }
      recommendationCoordinator.removeUser(worklist.getUser());
    }
  }

  /**
   *
   * @param worklist RemoteWorklist
   */
  public synchronized void addWorklist(RemoteWorklist worklist) {
    if (worklist != null) {
      if (this.match(worklist)) { // only if this user belongs to my team
        if (!this.worklists.contains(worklist)) {
          synchronized (worklists) {
            this.worklists.add(worklist);
          }
          this.recommendationCoordinator.addUser(worklist.getUser());
          this.initiate(worklist);
        }
      }
    }
  }

  public synchronized void reInitiate() {
    synchronized (worklists) {
      List<RemoteWorklist> remove = new ArrayList<RemoteWorklist> ();
      Iterator<RemoteWorklist> iterator = worklists.iterator();
      while (iterator.hasNext()) {
        RemoteWorklist worklist = iterator.next();
        if (!match(worklist)) {
          remove.add(worklist);
          closeAssignment(worklist);
        }
      }
      worklists.removeAll(remove);
    }
  }

  protected synchronized RemoteWorklist getWorklist(User user) {
    RemoteWorklist worklist = null;
    boolean found = false;
    synchronized (worklists) {
      Iterator<RemoteWorklist> iterator = worklists.iterator();
      while (iterator.hasNext() && !found) {
        worklist = iterator.next();
        found = worklist.getUser().equals(user);
      }
    }
    return found ? worklist : null;
  }

  /**
   *
   * @param worklist RemoteWorklist
   * @return boolean
   */
  public boolean match(RemoteWorklist worklist) {
    boolean ok = false;
    if (worklist != null) {
      User user = worklist.getUser();
      ok = assignment.getTeam().isMember(user);
    }
    return ok;
  }

  protected void initiate(RemoteWorklist worklist) {
    this.openAssignment(worklist);
    this.sendChangeState(worklist);
  }

  private void openAssignment(RemoteWorklist worklist) {
    IMessage open = MessageFactory.createOpenAssignementMessage(
        assignment, this.assignmentString);
    worklist.sendInformation(open);
  }

  private void changeAssignment(RemoteWorklist worklist) {
    IMessage change = MessageFactory.createChangeAssignmentMassage(
        assignment, this.assignmentString);
    worklist.sendInformation(change);
  }

  protected void sendChangeState(RemoteWorklist worklist) {
    sendChangeMessages(worklist, createChangeMessages(worklist));
  }

  protected void sendChangeMessages(RemoteWorklist worklist, List<IMessage>
      msgs) {
    if (worklist != null && msgs != null) {
      Iterator<IMessage> i = msgs.iterator();
      while (i.hasNext()) {
        worklist.sendInformation(i.next());
      }
    }
  }

  private List<IMessage> createChangeMessages(RemoteWorklist worklist) {
    IMessage state = this.getState(worklist);
    IMessage recommendation = this.getRecommendation(worklist);
    List<IMessage> msgs = new ArrayList<IMessage> ();
    if (recommendation != null) { // first recommendation
      msgs.add(recommendation);
    }
    if (state != null) { // then state
      msgs.add(state);
    }
    return msgs;
  }

  /**
   *
   */
  public synchronized void changeState() {
    this.createState();
    synchronized (worklists) {
      Iterator<RemoteWorklist> iterator = worklists.iterator();
      while (iterator.hasNext()) {
        RemoteWorklist worklist = iterator.next();
        this.sendChangeState(worklist);
      }
    }
  }

  /**
   *
   */
  public synchronized void changeModel() {
    synchronized (worklists) {
      Iterator<RemoteWorklist> iterator = worklists.iterator();
      while (iterator.hasNext()) {
        RemoteWorklist worklist = iterator.next();
        this.changeAssignment(worklist);
      }
    }
  }

  /**
   *
   * @param worklist RemoteWorklist
    * @return IMessage
    */
   private IMessage getState(RemoteWorklist worklist) {
     AssignmentState authorizatiton = this.authorize(this.getAssignmentState(),
         worklist.getUser());
     return MessageFactory.createStateAssignmentMassage(
         authorizatiton);
   }

  private IMessage getRecommendation(RemoteWorklist worklist) {
    IMessage msg = null;
    if (worklist != null) {
      Response result = recommendationCoordinator.
          getFullRecommendation();
      if (result != null) {
          msg = MessageFactory.
              messageRecommendationAssignment(assignment, result);
      }
    }
    return msg;
  }

  /**
   *
   * @param state AssignmentState
   * @param user RemoteWorklist
   * @return AssignmentState
   */
  private AssignmentState authorize(AssignmentState state, User user) {
     return state.authorize(user);
  }

  /**
   *
   * @return Assignment
   */
  public Assignment getAssignment() {
    return this.assignment;
  }

  /**
   *
   * @param path String
   */
  private void getAssignmentAndViewString(String path) {
    AssignmentViewBroker broker = XMLBrokerFactory.newAssignmentBroker(path);
    if (broker != null) {
      if (broker instanceof XMLAssignmentViewBroker) {
        XMLAssignmentViewBroker xmlBroker = (XMLAssignmentViewBroker) broker;
        assignmentString = XMLParser.toString(xmlBroker.
                                              getAssignmentAndView());
      }
    }
  }

  public void setAssignmentAndViewString(Assignment assignment,
                                         AssignmentModelView view) {
    AssignmentBroker broker = XMLBrokerFactory.newAssignmentBroker();
    if (broker != null) {
      if (broker instanceof XMLAssignmentViewBroker) {
        XMLAssignmentViewBroker xmlBroker = (XMLAssignmentViewBroker) broker;
        assignmentString = XMLParser.toString(xmlBroker.
                                              assignmentAndViewElement(
            assignment, view));
      }
    }
  }

  public boolean forAssignment(String id) {
    return assignment.getId() == Integer.parseInt(id);
  }

  /**
   *
   * @return boolean
   * @param secure boolean
   * @throws Exception
   */
  public synchronized boolean closeAssignment(boolean secure) {
    boolean ok = secure ? canComplete() : true;
    if (ok) {
      synchronized (worklists) {
        Iterator<RemoteWorklist> iterator = worklists.iterator();
        while (iterator.hasNext()) {
          RemoteWorklist worklist = iterator.next();
          this.closeAssignment(worklist);
        }
      }

    }
    return ok;
  }

  /**
   *
   * @param worklist boolean
   */
  public synchronized void closeAssignment(RemoteWorklist worklist) {
    worklist.sendInformation(MessageFactory.messageCloseAssignment(assignment));
  }

  /**
   *
   * @return boolean
   * @throws Exception
   */
  public boolean canComplete() {
     return true;
  }

  /**
   *
   * @param item String
   * @param event Event
   * @return WorkItemEvent
   */
  private void execute(WorkItem item, Event event) {
    execution.execute(item, event);
    if ( event.getType().equals(Event.Type.COMPLETED)) {
      item.writeData();
    }
  }

  /**
   *
   * @param item User
   * @param event String
   * @return WorkItemEvent
   */
  public synchronized boolean event(WorkItem item, Event event) {
    this.execute(item, event);
    boolean ok = false;
    if (event != null) { // accept
      ok = strategy.next(event);
      this.changeState();
    }
    return ok;
  }

  public void setRecommendationProvider(OperationalSupportClient recommendationProvider) {
    this.recommendationCoordinator = new RecommendationCoordinator(
        assignmentState,
        recommendationProvider);
  }

  public OperationalSupportClient getRecommendationProvider( ) {
    return recommendationCoordinator.getRecommendationProvider();
  }

  public void change(Assignment change, AssignmentModelView view) throws
      Throwable {
    this.setAssignment(change);
    this.setAssignmentAndViewString(this.assignment, view);
    this.changeModel();
    this.changeState();
  }


  public boolean active(int id) {
    boolean active = true;
    ConstraintDefinition constraint = assignment.constraintWithId(id);
    if (constraint != null && constraint instanceof Constraint) {
      active = ( (Constraint) constraint).isActive();
    }
    return active;
  }
}
