package nl.tue.declare.appl.framework.engine;

/**
 * <p>Title: DECLARE</p>
 *
 * <p>Description: </p>
 *
 * <p>Copyright: Copyright (c) 2006</p>
 *
 * <p>Company: TU/e</p>
 *
 * @author Maja Pesic
 * @version 1.0
 */
import java.net.*;
import java.util.*;

import org.processmining.models.operationalsupport.net.client.OperationalSupportClient;

import nl.tue.declare.appl.framework.FrameworkCoordinator;
import nl.tue.declare.control.*;
import nl.tue.declare.datamanagement.XMLWorkItemBroker;
import nl.tue.declare.domain.instance.*;
import nl.tue.declare.domain.organization.*;
import nl.tue.declare.execution.*;
import nl.tue.declare.execution.msg.*;
import nl.tue.declare.execution.msg.engine.response.*;
import nl.tue.declare.execution.msg.worklist.*;
import nl.tue.declare.graph.model.*;
import nl.tue.declare.logging.*;
import nl.tue.declare.utils.sockets.*;
import yawlservice.ExternalCase;

public class EngineCoordinator
    implements IRemoteWorklistListener, IRequestListener {

  private EngineGate requestGate; // for request-response communication with worklists, eg. WL_request_event -> ENG_response_accept_or_reject
  private EngineGate monitorGate; // for sending information to worklists, eg.,  ENG_change_state is sent to worklists without a request

  protected List<RemoteWorklist> worklists;
  private List<EngineAssignmentCoordinator> assignments;
  protected static EngineCoordinator instance = null;

  private int portRecommendation;
  private String hostRecommendation;
  private OperationalSupportClient recommendationProvider;
  /**
   *
   */
  protected EngineCoordinator() {
    super();
    this.assignments = Collections.synchronizedList(new ArrayList<EngineAssignmentCoordinator>());
    this.worklists = Collections.synchronizedList(new ArrayList<RemoteWorklist>());

    EngineInitializer ini = EngineInitializer.singleton(); // load settings for ini file
    int portInfo = ini.getPortInformation(); // read engine information port from ini file
    int portReq = ini.getPortRequest(); // read engine request port from ini file

    portRecommendation = ini.getPortRecommendation(); // read recommendation proxy port from ini file
    hostRecommendation = ini.getHostRecommendation(); // read recommendation proxy host from ini file
    if (hostRecommendation == null) {
      hostRecommendation = SocketConnection.LOCAL_HOST;
    }
    //recommendationProvider = new RecommendationProxy(this.hostRecommendation,      this.portRecommendation);
    recommendationProvider = new  OperationalSupportClient(this.hostRecommendation,  this.portRecommendation);
    
    this.requestGate = new EngineGate() { // create request gate
      public void connected(IWorklistRemoteProxy proxy) {
        proxy.setRequestListener(EngineCoordinator.this);
      }

      protected IWorklistRemoteProxy createProxy(Socket connection) {
        return new ResponseWorklistProxy(connection);
      }
    };
    if (portReq > 0) { // if loaded
      this.requestGate.setPort(portReq); // set port
    }

    this.monitorGate = new EngineGate() { // create monitor gate
      public void connected(IWorklistRemoteProxy proxy) {
        proxy.setRequestListener(EngineCoordinator.this);
      }

      protected IWorklistRemoteProxy createProxy(Socket connection) {
        return new InformationWorklistProxy(connection);
      }
    };
    if (portInfo > 0) { // if loaded
      this.monitorGate.setPort(portInfo); // set port
    }
  }

  public void initiate() throws Exception {
    this.requestGate.init();
    this.monitorGate.init();
  }

  /**
   *
   * @return EngineCoordinator
   */
  public static EngineCoordinator singleton() {
    if (instance == null) {
      instance = new EngineCoordinator();
    }
    return instance;
  }

  /**
   *
   * @param id int
   * @param user User
   * @param proxy IWorklistRemoteProxy
   * @return int
   */
  public int connected(int id, User user, IWorklistRemoteProxy proxy) {
    int newId = 0;
    if (proxy != null) {
      RemoteWorklist worklist = getWorklist(id, user);
      if (proxy instanceof ResponseWorklistProxy) {
        worklist.addResponseChannel( (ResponseWorklistProxy) proxy);
      }
      if (proxy instanceof InformationWorklistProxy) {
        worklist.addInformationChannel( (InformationWorklistProxy) proxy);
      }
      if (worklist.connected()) {
        initiateWorklist(worklist);
      }
      newId = worklist.getId();
    }
    return newId;
  }

  public boolean logout(int id, User user) {
    boolean ok = true;
    RemoteWorklist worklist = getWorklist(id, user);
    if (worklist != null) {
      disconnected(worklist);
    }
    return ok;
  }

  private int nextWorklistId() {
    int id = 1;
    synchronized (worklists) {
      if (worklists.size() > 0) {
        id = worklists.get(worklists.size() - 1).getId() + 1;
      }
    }
    return id;
  }

  private RemoteWorklist getWorklist(int id, User user) {
    RemoteWorklist worklist = null;
    synchronized (worklists) {
      Iterator<RemoteWorklist> iterator = worklists.iterator();
      boolean found = false;
      while (iterator.hasNext() && !found) {
        worklist = iterator.next();
        found = worklist.getId() == id && worklist.forUser(user);
      }
      if (!found) {
        worklist = new RemoteWorklist(nextWorklistId(), user, this);
        synchronized (worklists) {
          worklists.add(worklist);
        }
      }
    }
    return worklist;
  }

  public Collection<EngineAssignmentCoordinator> getInstances(
      EngineAssignmentCoordinator
      ac) {
    Collection<EngineAssignmentCoordinator>
        instances = new ArrayList<EngineAssignmentCoordinator> ();
    synchronized (this.assignments) {
      for (int i = 0; i < assignments.size(); i++) {
        EngineAssignmentCoordinator curr = assignments.get(i);
        if (curr.sameModel(ac)) {
          instances.add(curr);
        }
      }
    }
    return instances;
  }

  /**
   *
   * @param assignmentCoordinator AssignmentCoordinator
   * @param scheduledAt Double
   */
  public synchronized void add(EngineAssignmentCoordinator
                               assignmentCoordinator) {
    if (assignmentCoordinator != null) {
      assignmentCoordinator.setRecommendationProvider(this.
          getRecommendationProxy());
      synchronized (assignments) {
        this.assignments.add(assignmentCoordinator);
      }
      this.initiateAssignment(assignmentCoordinator);
    }
  }

  private OperationalSupportClient getRecommendationProxy() {
    return recommendationProvider;
  }

  /**
   *
   * @param assignmentCoordinator AssignmentCoordinator
   */
  protected void initiateAssignment(EngineAssignmentCoordinator
                                    assignmentCoordinator) {
    synchronized (worklists) {
      for (int i = 0; i < this.worklists.size(); i++) {
        RemoteWorklist worklist = this.worklists.get(i);
        this.match(assignmentCoordinator, worklist);
      }
    }
  }

  /**
   *
   * @param worklist RemoteWorklist
   */
  protected void initiateWorklist(RemoteWorklist worklist) {
    synchronized (assignments) {
      for (int i = 0; i < this.assignments.size(); i++) {
        EngineAssignmentCoordinator assignmentCoordinator = this.assignments.
            get(i);
        match(assignmentCoordinator, worklist);
      }
    }
  }

  /**
   *
   * @param coordinator AssignmentCoordinator
   * @param worklist RemoteWorklist
   * @return boolean
   */
  private boolean match(EngineAssignmentCoordinator coordinator,
                        RemoteWorklist worklist) {
    boolean ok = false;
    if (coordinator != null && worklist != null) {
      ok = coordinator.match(worklist);
      if (ok) {
        coordinator.addWorklist(worklist);
      }
    }
    return ok;
  }

  /**
   *
   * @param worklist RemoteWorklist
   */
  public void disconnected(RemoteWorklist worklist) {
    synchronized (worklists) {
      worklists.remove(worklist);
    }

    synchronized (assignments) {
      for (int i = 0; i < this.assignments.size(); i++) {
        EngineAssignmentCoordinator assignmentCoordinator = this.assignments.
            get(i);
        assignmentCoordinator.removeWorkList(worklist);
      }
    }
  }

  /**
   *
   * @param id String
   * @return AssignmentCoordinator
   */
  public EngineAssignmentCoordinator getAssignmentCoordinator(String id) {
    boolean found = false;
    int i = 0;
    EngineAssignmentCoordinator coordinator = null;
    synchronized (assignments) {
      while ( (!found) && (i < this.assignments.size())) {
        coordinator = this.assignments.get(i++);
        found = coordinator.forAssignment(id);
      }
    }
    return found ? coordinator : null;
  }

  /**
   *
   * @param assignment String
   * @return AssignmentCoordinator
   */
  public EngineAssignmentCoordinator getAssignmentCoordinator(Assignment
      assignment) {
    boolean found = false;
    int i = 0;
    EngineAssignmentCoordinator coordinator = null;
    synchronized (assignments) {
      while ( (!found) && (i < this.assignments.size())) {
        coordinator = this.assignments.get(i++);
        found = coordinator.getAssignment() == assignment;
      }
    }
    return found ? coordinator : null;
  }

  /**
   *
   * @param assignment assignment that sould be closed
   * @param secure true close iff assignment in state to be closed; false close
   *   without checking the state of assinment
   * @throws Exception
   * @return boolean
   */
  public synchronized boolean complete(Assignment assignment, boolean secure) {
    boolean ok = false;
    EngineAssignmentCoordinator coordinator = this.getAssignmentCoordinator(
        assignment);
    if (coordinator != null) {
      ok = coordinator.closeAssignment(secure);
      if (ok) {
        synchronized (assignments) {
          this.assignments.remove(coordinator);
        }
      }
    }
    return ok;
  }

  /**
   *
   * @param assignment String
   * @throws Exception
   * @return boolean
   */
  public boolean canComplete(Assignment assignment) throws Exception {
    EngineAssignmentCoordinator coordinator = this.getAssignmentCoordinator(
        assignment);
    boolean ok = false;
    if (coordinator != null) {
      ok = coordinator.canComplete();
    }
    return ok;
  }

  /**
   *
   * @param item WorkItem
   * @param event String
   * @return WorkItemEvent
   */
  public void eventRequest(WorkItem item, Event event) {
    EngineAssignmentCoordinator coordinator = this.getAssignmentCoordinator(
        Integer.
        toString(item.getActivity().getAssignment().getId()));
    if (coordinator != null) {
      coordinator.event(item, event);
    }
  }

  /**
   * close
   */
  public void close() {
    ProcessLogWriter.singleton().finish();
  }

  /* public void change(AssignmenCoordinator coordinator){
     coordinator.reInitiate();
     initiateAssignment(coordinator);
   }*/

  public synchronized void change(Assignment original, Assignment change,
                                  AssignmentModelView view) throws Throwable {
    EngineAssignmentCoordinator coordinator = getAssignmentCoordinator(original);
    if (coordinator != null) {
      coordinator.change(change, view);
      coordinator.reInitiate();
      initiateAssignment(coordinator);
    }
  }

  public synchronized void change(EngineAssignmentCoordinator coordinator,
                                  Assignment change, AssignmentModelView view) throws
      Throwable {
    if (coordinator != null) {
      coordinator.change(change, view);
      coordinator.reInitiate();
      initiateAssignment(coordinator);
    }
  }

  /**
   *
   * @param message AssignmentCoordinator
   */
  protected void sendToAll(IMessage message) {
    synchronized (worklists) {
      for (int i = 0; i < this.worklists.size(); i++) {
        RemoteWorklist worklist = this.worklists.get(i);
        worklist.sendInformation(message);
      }
    }
  }

  public IMessage request(IMessage msg, IWorklistRemoteProxy proxy) {
    if (msg == null) {
      return new UnknownRequest();
    }
    if (msg instanceof AbstractLoggingMessage) { // USER

      if (msg instanceof MessageLogIn) { // log in
        return this.logInMessage( (AbstractLoggingMessage) msg, proxy);
      }
      else if (msg instanceof MessageLogOut) { // log out
        return this.logOutMesage( (AbstractLoggingMessage) msg);
      }
    }
    else if (msg instanceof AssignmentMessage) { // ASSIGNMENT
      EngineAssignmentCoordinator coordinator = this.getAssignmentCoordinator( ( (
          AssignmentMessage) msg).id());
      if (msg instanceof MessageCloseAssignment) { // close assignment
        return this.closeMessage(coordinator);
      }
      else if (msg instanceof WorkItemMessage) { // WORK ITEM
        WorkItemMessage wimsg = (WorkItemMessage) msg;
        if (wimsg instanceof RequestEventMessage) { // REQUEST event
          return this.requestEventMessage(wimsg, coordinator, proxy.getUser());
        }
        else if (wimsg instanceof ForwardEventMessage) { // FORWARD event
          return this.forwardEventMessage((ForwardEventMessage)wimsg, coordinator, proxy.getUser());
        }
      }

    }

    return new UnknownRequest();
  }

  private IMessage logInMessage(AbstractLoggingMessage msg,
                                IWorklistRemoteProxy proxy) {
    IMessage response = null;
    User user = Control.singleton().getOrganization().getUserWithUserName(msg.
        getUsername());
    if (user == null) {
      response = new EngineLogInReject("Username " + msg.getUsername() +
                                       "does not exist.");
    }
    else
    if (!user.checkPasswod(msg.getPassword())) {
      response = new EngineLogInReject("Password is not correct.");
    }
    else {
      int id = this.connected(Integer.parseInt(msg.getId()), user, proxy);
      //response = new EngineLogInAccept(Integer.toString(id));
      response = new EngineLogInAccept(Integer.toString(id));
    }
    return response;
  }

  private IMessage logOutMesage(AbstractLoggingMessage msg) {
    IMessage response = null;
    User user = Control.singleton().getOrganization().getUserWithUserName(msg.
        getUsername());
    if (user != null) {
      if (logout(Integer.parseInt(msg.getId()), user)) {
        response = new ApprovedRequest();
      }
      else {
        response = new RejectedRequest();
      }
    }
    if (response == null) {
      response = new UnknownRequest();
    }
    return response;
  }

  /**
   * response
   *
   * @return List
   * @todo Implement this
   *   nl.tue.declare.execution.msg.engine.handle.IEngineMessageHandler method
   */
  private IMessage closeMessage(EngineAssignmentCoordinator cooordinator) {
    Assignment assignment = cooordinator.getAssignment();
    String reason = "";
    if (assignment != null) {
      boolean ok = false;
      try {
        ok = FrameworkCoordinator.singleton().complete(assignment);
      }
      catch (Exception ex) {
        reason = ex.getMessage();
      }
      if (ok) {
        return new EngineCloseAssignmentAccept(assignment);
      }
      else {
        return new EngineCloseAssignmentReject(assignment,
                                               "Assignment cannot be closed at this moment. " +
                                               reason);
      }
    }
    else {
      return new UnknownRequest();
    }
  }

  /**
   * response
   *
   * @return List
   * @todo Implement this
   *   nl.tue.declare.execution.msg.engine.handle.IEngineMessageHandler method
   */
  public IMessage forwardEventMessage(ForwardEventMessage msg,
                                      EngineAssignmentCoordinator coordinator,
                                      User user) {
    WorkItem item = this.createItem(coordinator.getAssignment(), user, msg);
    Event event = new Event(item.getUser(), item.getActivity(),Event.Type.valueOf(msg.getEventName()));
    ExternalCase ec = new  ExternalCase();
    ec.fromString(msg.getExternalCase());
    try {
      if (item != null && event != null && coordinator != null) {
        coordinator.event(item, event);
        if (this.handleForward(item,ec)) {
          // add here my attempt to forward to YAWL
            return new EngineEventAccept(item,event);
        }
        else {
          return new EngineEventReject(item, event, "");
        }
      }
      else {
        return new UnknownRequest();
      }
    }
    catch (Throwable t) {
      return new EngineEventReject(item, event, "");
    }
  }

  private boolean handleForward(WorkItem item, ExternalCase ec){
      return FrameworkCoordinator.singleton().forwardWorkItemToYawl(item,ec);
  }

  /**
   * response
   *
   * @return List
   * @todo Implement this
   *   nl.tue.declare.execution.msg.engine.handle.IEngineMessageHandler method
   */
  public IMessage requestEventMessage(WorkItemMessage msg,
                                      EngineAssignmentCoordinator coordinator,
                                      User user) {
    WorkItem item = this.createItem(coordinator.getAssignment(), user, msg);
    Event event = new Event(item.getUser(),item.getActivity(),
                                                 Event.Type.valueOf(msg.getEventName()));
    try {
      if (item != null && event != null && coordinator != null) {
        boolean ok = coordinator.event(item, event);
        if (ok) {
          return new EngineEventAccept(item, event);
        }
        else {
          return new EngineEventReject(item, event, "");
        }
      }
      else {
        return new UnknownRequest();
      }
    }
    catch (Throwable t) {
    t.printStackTrace();	
      return new EngineEventReject(item, event, "");
    }
  }

  /**
   *
   * @param activityID String
   * @param itemID String
   * @param xml String
   * @return WorkItem
   */
  private WorkItem createItem(Assignment assignment, User user,
                              WorkItemMessage msg) {
    WorkItem workItem = null;
    if (assignment != null) {
      Activity activity = (Activity) assignment.activityDefinitionWithId(
          Integer.
          parseInt(msg.getActivityId()));
      int id = Integer.parseInt(msg.getItemId());
      workItem = new WorkItem(id, activity, user);

      XMLWorkItemBroker broker = new XMLWorkItemBroker(workItem);
      broker.readDocumentString(msg.getXML());
      broker.fromElement();
    }
    return workItem;
  }
}
