package nl.tue.declare.appl.framework.recommendation;

import java.util.*;

import org.deckfour.xes.extension.std.XLifecycleExtension;
import org.deckfour.xes.model.XEvent;
import org.processmining.models.operationalsupport.AvailableItem;
import org.processmining.models.operationalsupport.Request;
import org.processmining.models.operationalsupport.Response;
import org.processmining.models.operationalsupport.net.client.OperationalSupportClient;

import nl.tue.declare.domain.instance.*;
import nl.tue.declare.domain.organization.*;
import nl.tue.declare.execution.*;
import nl.tue.declare.logging.*;
import nl.tue.declare.recommendation.*;
import nl.tue.declare.utils.prom.*;

public class RecommendationCoordinator {

	private OperationalSupportClient recommendationProxy;
	private AssignmentState state;
	private Set<User> availableUsers;

	public RecommendationCoordinator(AssignmentState state,
			OperationalSupportClient recommendationProxy) {
		super();
		this.state = state;
		this.recommendationProxy = recommendationProxy;
		this.availableUsers = Collections.synchronizedSet(new HashSet<User>());
	}

	public void addUser(User user) {
		this.availableUsers.add(user);
	}

	public void removeUser(User user) {
		availableUsers.remove(user);
	}

	public OperationalSupportClient getRecommendationProvider() {
		return recommendationProxy;
	}

	/**
	 * 
	 * @return RecommendationQuery
	 */
	private Request fullQuery() {
		Assignment assignment = state.getAssignment();
		String qid = Long.toHexString(System.currentTimeMillis()); // query id
		String pid = state.getAssignment().getName(); // process id
		String piid = Integer.toString(assignment.getId()); // process instance
															// id
		Request query = new Request(qid, pid, piid);
		for (AssignmentState.EventState eventState : state.getPossibleEvents()) {
			Event event = eventState.event();
			String task = event.getActivity().getName();
			XLifecycleExtension.StandardModel type = ProM.getType(event);
			AvailableItem item = new AvailableItem(task, type);
			synchronized (availableUsers) {
				for (User user : availableUsers) { // available resources
					item.addAuthorizedResource(RecommendationUtils
							.getUser(user));
				}
			}
			// These are the LogEvents which are currently enabled
			query.addAvailableItem(item);

		}

		// add incomplete log
		AssignmentLog log = ProcessLogWriter.singleton().getAssignmentLog(
				assignment);
		for (XEvent event: log.log()){
			query.addEvent(event);
		}
		return query;
	}

	/**
	 * 
	 * @param query
	 *            RecommendationQuery
	 * @return RecommendationResult
	 */
	private Response getRecommendation(Request query) {
		try {
			return recommendationProxy.getResponse(query);
		} catch (Exception ex) {
			// No recommendation received. Probably and error in the
			// communication
			return null;
		}
	}

	/**
	 * 
	 * @return RecommendationResult
	 */
	public Response getFullRecommendation() {
		Request query = this.fullQuery();
		return getRecommendation(query);
	}
}
