package nl.tue.declare.appl.worklist;

/**
 * <p>Title: DECLARE</p>
 *
 * <p>Description: </p>
 *
 * <p>Copyright: Copyright (c) 2006</p>
 *
 * <p>Company: TU/e</p>
 *
 * @author Maja Pesic
 * @version 1.0
 */
import java.util.*;

import java.awt.*;
import javax.swing.*;

import org.processmining.models.operationalsupport.Response;

import nl.tue.declare.appl.util.swing.*;
import nl.tue.declare.appl.worklist.gui.*;
import nl.tue.declare.domain.instance.*;
import nl.tue.declare.domain.instance.Event;
import nl.tue.declare.domain.organization.*;
import nl.tue.declare.execution.*;
import nl.tue.declare.execution.msg.IMessage;
import nl.tue.declare.execution.msg.MessageFactory;
import nl.tue.declare.execution.msg.engine.response.EngineEventAccept;
import nl.tue.declare.execution.msg.engine.response.EngineEventReject;
import nl.tue.declare.graph.assignment.*;

public class WorklistAssignmentCoordinator  implements IAssignmentExecutionListener {
	//extends		AbstractAssignmentCoordinator {
	  protected Assignment assignment;
	  protected AssignmentView view = null;
	  protected Collection<WorkItem> items;
	  protected HistoryCoordinator history;
	  protected IListener listener = null;
	  private User user = null;

	private JFrame frame = null;
	private JPanel panel = null;
	private RecommendationPanel recommendation;
	private ActivitiesPane activitiesPane = null;
	private JSplitPane split;

	private JPanel graphPanel = null;

	FrmWarning frmWarning;

	public WorklistAssignmentCoordinator(Assignment assignment, JFrame frame,
			User user) {
		super();
	    this.assignment = assignment;
	    this.user = user;
	    items = new ArrayList<WorkItem>();
	    history = new HistoryCoordinator(getAssignment());
	    Thread t = new Thread(history);
	    t.start();
	    
		this.frame = frame;
		panel = new JPanel(new BorderLayout());
		activitiesPane = new ActivitiesPane();
		recommendation = new RecommendationPanel();
		recommendation.setVisible(false);
		frmWarning = new FrmWarning(getMainFrame(), panel);
		prepareUI();
	}

	private JFrame getMainFrame() {
		return frame;
	}

	/**
   *
   */
	private void prepareUI() {
		graphPanel = new TPanel(new BorderLayout(), getAssignment().toString());

		JScrollPane graph = new JScrollPane(graphPanel);
		JScrollPane activity = new JScrollPane(activitiesPane);

		split = new TSplitPane(JSplitPane.HORIZONTAL_SPLIT, graph,
				recommendation);
		// splitUp.setResizeWeight(0.5);

		JSplitPane splitPane = new TSplitPane(JSplitPane.VERTICAL_SPLIT,
				split, activity);
		splitPane.setDividerLocation(500);

		Dimension minimumSize = new Dimension(100, 50);
		graph.setMinimumSize(minimumSize);
		activity.setMinimumSize(minimumSize);

		panel.add(splitPane);
	}

	public void setView(AssignmentView view) {
	    this.view = view;
	    if (view != null) {
	      view.setListener(this);
	    }
		graphPanel.removeAll();
		if (view != null) {
			graphPanel.add(view.getGraph(), BorderLayout.CENTER);
		}
	}

	/**
	 * 
	 * @return JPanel
	 */
	public JPanel getWorkPanel() {
		view.getGraph().updateUI();
		activitiesPane.updateUI();
		return panel;
	}

	/**
   *
   */
	public void stateChanged() {
		view.updateStateView();
		AssignmentState state = view.getState();
		activitiesPane.disableAll();
		view.updateStateView();
		for (int i = 0; i < state.eventCount(); i++) {
			AssignmentState.EventState eventState = state.eventAt(i);
			Event event = eventState.event();
			if (event.getType().equals(Event.Type.COMPLETED)) {
				Activity activity = state.getAssignment()
						.getActivityForDefinition(event.getActivity());
				activitiesPane.setEnabledComplete(activity, true);
			}
			if (event.getType().equals(Event.Type.CANCELLED)) {
				Activity activity = state.getAssignment()
						.getActivityForDefinition(event.getActivity());
				activitiesPane.setEnabledCancel(activity, true);
			}
		}
		panel.repaint();
	}

	public void processRecommendation(Response result) {
		if (result != null) {
			recommendation.setRecommendationResult(result);
			if (!recommendation.isVisible()) {
				recommendation.setVisible(true);
				int location = split.getMaximumDividerLocation()
						- ((split.getMaximumDividerLocation() - split
								.getMinimumDividerLocation()) / 3);
				split.setDividerLocation(location);
			}
		} else {
			recommendation.setVisible(false);
		}
	}

	private boolean violate(String msg, Collection<Constraint> constraints) {
		frmWarning.fill(msg, constraints);
		return frmWarning.showCentered();
	}


		  public void setAssignment(Assignment assignment) {
		    this.assignment = assignment;
		    history.setAssignment(assignment);
		  }

		  public void setListener(IListener l) {
		    this.listener = l;
		  }

		  public void change(Assignment assignment, AssignmentView view) {
		    this.setAssignment(assignment);
		    this.setView(view);
		  }

		  public HistoryCoordinator getHistoryCoordinator() {
		    return this.history;
		  }

		  /**
		   *
		   * @return String
		   */
		  public String toString() {
		    return this.getAssignment().toString();
		  }

		  /**
		   *
		   * @return AssignmentState
		   */
		  public AssignmentState getState() {
		    return view.getState();
		  }

		  /**
		   *
		   * @return AssignmentView
		   */
		  public AssignmentView getView() {
		    return view;
		  }

		  /**
		   *
		   * @return Assignment
		   */
		  public Assignment getAssignment() {
		    //return getState().getAssignment();
		    return this.assignment;
		  }

		  /**
		   *
		   * @return AssignmentState
		   */
		  public AssignmentState getAssignmentState() {
		    return this.getState();
		  }


		  /**
		   *
		   * @return int
		   */
		  public int hashCode() {
		    return this.getAssignment().hashCode();
		  }

		  /**
		   *
		   * @return User
		   */
		  public User getUser() {
		    return this.user;
		  }

		  /**
		   *
		   * @param WorkItemEvent event
		   */
		  public void forwardActivity(WorkItem wi, Event event) {
		    if (event != null && listener != null) {
		          Activity activity = wi.getActivity();
		          // add parameters
		          IMessage response = listener.executeRequest(MessageFactory.eventForward(wi,event,activity.getExternalCase()));
		          if (response instanceof EngineEventReject) {
		      		MessagePane.inform(panel,
		    				"At the moment it is not possible to start/forward activity "
		    						+ activity);
		      }
		    }
		  }

		  /**
		   *
		   * @param activity Activity
		   */
		  public void startActivity(Activity activity) {
		    if (activity != null) {
		      Event event = new Event(this.getUser(), activity, Event.Type.STARTED);
		      if (this.view.getState().containsEvent(event)) {
		        // check for violations
		        if (this.warn(event)) {
		          WorkItem wi = new WorkItem(0, activity, getUser());
		          if (activity.isExternal()) { // if this is an external activity then forward it
		            this.forwardActivity(wi, event);
		          }
		          else { // if it is not an external activity just start it
		            WorkItem workItem = executeWorkItem(wi,event);
		            if (workItem != null) {
		    		    this.items.add(wi);
		    		    activitiesPane.addActivity(wi, this);
		    		    this.stateChanged();
		            }
		            else {
		        		MessagePane.inform(panel,
		        				"At the moment it is not possible to start activity "
		        						+ activity);
		            }
		          }
		        }
		      }
		    }
		  }

		  protected WorkItem executeWorkItem(WorkItem wi, Event event) {
		    IMessage response = executeEvent(wi, event);
		    WorkItem workItem = null;
		    if (response instanceof EngineEventAccept) {
		      //workItem = execute(wi.getActivity(), event,( (EngineEventAccept) response));
		    	workItem =  ( (EngineEventAccept) response).createItem(getUser(), wi.getActivity());
		    }
		    return workItem;
		  }



		  protected IMessage executeEvent(WorkItem wi, Event event) {
		    if (event != null) {
//		      AssignmentState.EventState eventState = getState().eventState(event. getEvent());
		      if ( /*eventState != null && */listener != null) {
		        // ask the engine here
		        return listener.executeRequest(MessageFactory.eventRequest(wi,event));
		      }
		    }
		    return null;
		  }

		  /**
		   *
		   * @param event AbstractEvent
		   * @return boolean
		   */
		  private boolean warn(Event event) {
		    boolean violate = true;
		    AssignmentState.EventState eventState = getState().eventState(event);

		    if (eventState != null) {
		      // check if some constraints would be violated with this event
		      if (eventState.violatedCout() > 0) {
		        Collection<Constraint> constraints = new ArrayList<Constraint>();

		        // loop through all violated constraints to inform the user
		        for (int i = 0; i < eventState.violatedCout(); i++) {
		          Constraint constraint = eventState.violatesAt(i);
		          if (!constraint.getMandatory()) {
		            constraints.add(constraint);
		          }
		        }
		        if (constraints.size() > 0) {
		    		String msg = "Event \"" + event.getProposition() + "\" will violate "
					+ constraints.size() + " constraint"
					+ (constraints.size() == 1 ? "." : "s.");
		          violate = this.violate(msg, constraints);
		        }
		      }
		    }
		    return violate;
		  }


		  /**
		   *
		   * @param workItem WorkItem
		   */
		  public void completeActivity(WorkItem workItem) {
		    if (workItem != null) {
		      Event event = new Event(this.getUser(),
		                                               workItem.getActivity(), Event.Type.COMPLETED);
		      boolean canComplete =  this.getAssignmentState().containsEvent(event);
		      if (!canComplete) {
		  		MessagePane.inform(panel,
						"At the moment it is not possible to complete activity "
								+ workItem.getActivity());
		      }
		      else {
		        boolean dataOk = this.activitiesPane.readData(workItem);
		        if (dataOk) {
		          if (this.warn(event)) {
		            if (executeEvent(workItem, event) != null) {
		              this.removeWorkItem(workItem);
		            }
		            else {
		        		MessagePane.inform(panel,
		        				"At the moment it is not possible to complete activity "
		        						+ workItem.getActivity());
		            }
		          }
		        }
		      }
		    }
		  }

		  /**
		   *
		   * @param item WorkItem
		   */
		  private void removeWorkItem(WorkItem item) {
		    this.items.remove(item);
		    this.activitiesPane.removeWorkItem(item);
		  }


		  /**
		   *
		   * @param workItem WorkItem
		   */
		  public void cancelActivity(WorkItem workItem) {
		    if (workItem != null) {
		      Event event = new Event(this.getUser(),
		                                              workItem.getActivity(), Event.Type.CANCELLED);
		      if (this.warn(event)) {
		        if (this.executeWorkItem(workItem, event) != null) {
		          this.removeWorkItem(workItem);
		        }
		        else {
		    		MessagePane.inform(panel,
		    				"At the moment it is not possible to cancel activity "
		    						+ workItem.getActivity());
		        }
		      }
		    }
		  }

		  /**
		   *
		   * @param id int
		   * @return WorkItem
		   */
		  public WorkItem getWorkItem(int id) {
		    WorkItem item = null;
		    Iterator<WorkItem> iterator = items.iterator();
		    boolean found = false;
		    while (iterator.hasNext() && !found) {
		      item = iterator.next();
		      found = item.getId() == id;
		    }
		    return found ? item : null;
		  }


		  public boolean canStop() {
		    return this.getAssignmentState().getState().equals(State.SATISFIED);
		  }

		  /**
		   *
		   * @return boolean
		   */
		  boolean warnClosing() {
		    boolean violate = true;
		    Collection<Constraint> constraints = new ArrayList<Constraint>();

		    // loop through all violated constraints to inform the user
		    for (int i = 0; i < getAssignment().constraintDefinitionsCount(); i++) {
		      Constraint constraint = getAssignment().constraintAt(i);
		      if (!constraint.getMandatory()) {
		        if (constraint.getState().equals(State.VIOLATED_TEMPORARY)) {
		          constraints.add(constraint);
		        }
		      }
		    }
		    if (constraints.size() > 0) {		    	
				String msg = "Closing assignment \"" + this.getAssignment()
				+ "\" will violate " + constraints.size() + " constraint"
				+ (constraints.size() == 1 ? "." : "s.");
		        violate = this.violate(msg, constraints);	
		    }
		    return violate;
		  }

		  State state() {
		    return this.getAssignment().getState();
		  }

		  public interface IListener {
		    void closeAssignment(WorklistAssignmentCoordinator coordinator);

		    IMessage executeRequest(IMessage request);
		  }
}
