package nl.tue.declare.appl.worklist;

/**
 * <p>Title: DECLARE</p>
 *
 * <p>Description: </p>
 *
 * <p>Copyright: Copyright (c) 2006</p>
 *
 * <p>Company: TU/e</p>
 *
 * @author Maja Pesic
 * @version 1.0
 */

import java.awt.*;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.ArrayList;
import java.util.List;

import org.processmining.models.operationalsupport.Response;
import org.processmining.models.operationalsupport.xml.OSXMLConverter;


import nl.tue.declare.appl.util.ErrorMessage;
import nl.tue.declare.appl.util.swing.*;
import nl.tue.declare.appl.worklist.gui.*;
import nl.tue.declare.datamanagement.XMLBrokerFactory;
import nl.tue.declare.datamanagement.assignment.XMLAssignmentViewBroker;
import nl.tue.declare.datamanagement.assignment.XMLAssignmnetStateBroker;
import nl.tue.declare.domain.instance.*;
import nl.tue.declare.domain.model.AssignmentModel;
import nl.tue.declare.domain.organization.Administrator;
import nl.tue.declare.domain.organization.User;
import nl.tue.declare.execution.AssignmentState;
import nl.tue.declare.execution.msg.AssignmentMessage;
import nl.tue.declare.execution.msg.IMessage;
import nl.tue.declare.execution.msg.MessageFactory;
import nl.tue.declare.execution.msg.engine.msg.ChangeAssignmentMessage;
import nl.tue.declare.execution.msg.engine.msg.CloseAssignmentEngineMessage;
import nl.tue.declare.execution.msg.engine.msg.HistoryAssignmentMessage;
import nl.tue.declare.execution.msg.engine.msg.OpenAssignmentMessage;
import nl.tue.declare.execution.msg.engine.msg.RecommendationAssignmentMessage;
import nl.tue.declare.execution.msg.engine.msg.StateAssignmentMessage;
import nl.tue.declare.execution.msg.engine.response.EngineCloseAssignmentAccept;
import nl.tue.declare.execution.msg.engine.response.EngineCloseAssignmentReject;
import nl.tue.declare.execution.msg.engine.response.EngineLogInAccept;
import nl.tue.declare.execution.msg.engine.response.EngineLogInReject;
import nl.tue.declare.execution.msg.worklist.MessageLogIn;
import nl.tue.declare.graph.assignment.AssignmentView;

public class WorklistCoordinator
implements WindowListener, IRemoteEngineListener,
IWorklistFrameListener, WorklistAssignmentCoordinator.IListener {

	  protected RemoteEngineProxy proxy;

	  private LogInFrame logInFrame;

	  protected User user;
	  private int id = -1;

	  protected List<WorklistAssignmentCoordinator> coordinators;

	  private Object readLock = new Object();
	  private OSXMLConverter converter;
	
  private WorklistFrame frame;

  public WorklistCoordinator() {
    super();
    this.coordinators = new ArrayList<WorklistAssignmentCoordinator>();
    converter = new OSXMLConverter();
    ini();
  }

  /**
   *
   * @param object Object
   */
  public void assignmentSelected(Object object) {
    if (object != null) {
      if (object instanceof WorklistAssignmentCoordinator) {
        WorklistAssignmentCoordinator assignment = (WorklistAssignmentCoordinator) object;
        frame.addWorkPanel(assignment.getWorkPanel());
      }
    }
  }



  protected void begin() {
    // startup the worklist frame
    frame = new WorklistFrame(user.getUserName());
    frame.addWindowListener(this);
    frame.addWorklistFrameListener(this);
    frame.showMe();
  }

  protected void added(WorklistAssignmentCoordinator coordinator) {
    if (coordinator != null) {
      frame.addAssignment(coordinator);
    }
  }

  protected boolean close(WorklistAssignmentCoordinator coordinator) {
    if (coordinator != null) {
      if (!coordinator.state().equals(State.SATISFIED)) {
        MessagePane.inform(this.frame,
                           "Cannot close assignment " +
                           coordinator.getAssignment().toString() +
                           " because some of the constraints are violated.");
        return false;
      }
      else {
        String msg = "Are you sure you want to close assignment " +
            coordinator.getAssignment() + "?";
        return MessagePane.ask(this.frame, msg);
      }
    }
    return true;
  }

  protected void closed(WorklistAssignmentCoordinator coordinator) {
    if (coordinator != null) {
      this.frame.removeAssignment(coordinator);
      this.frame.repaint();
    }
  }

  protected void changed(WorklistAssignmentCoordinator coordinator) {
    if (coordinator != null) {
      Color color = coordinator.getView().getColor();
      boolean enabled = coordinator.getState().enabled();
      this.frame.setState(coordinator, enabled, color);
      coordinator.stateChanged();
    }
  }

  
  public void finish() {
    this.logOut();
  }
  

  public void setId(int id) {
    this.id = id;
  }

  /**
   *
   * @return User
   */
  public User getUser() {
    return user;
  }

  public int getId() {
    return id;
  }

  /**
   *
   */
  private void ini() {
    logInFrame = new LogInFrame("Worklist", this,
                                Administrator.singleton().getUserName(),
                                Administrator.singleton().getPassword());
    logInFrame.showMe();
  }


  /**
   *
   * @param username String
   * @param password String
   */
  public void logIn(String username, String password) {

    SettingsCoordinator ini = SettingsCoordinator.singleton(); // initialize form ini file
    String host = ini.getHost(); // load engine host from in file
    int portInfo = ini.getPortInfo(); // load engine information port from ini file
    int portReq = ini.getPortReqest(); // load engine request port from ini file

    // create a proxy for connection with the engine
    proxy = new RemoteEngineProxy(username, password, portInfo, portReq, host);

    proxy.addListener(this);
    try {
      // conect the proxy to the engine
      proxy.connect();
      // create a request message for logging in
      MessageLogIn msgLogIn = new MessageLogIn(Integer.toString(id), username,
                                               password);

      // get the response
      IMessage response = proxy.request(msgLogIn);
      if (response != null) { // if no error
        if (response instanceof EngineLogInAccept) { // the logging in is ACCEPTED
          id = Integer.parseInt( ( (EngineLogInAccept) response).getId());

          proxy.monitorLogIn(new MessageLogIn(Integer.toString(id), username,
                                              password));
          // hide the loggingin frame
          logInFrame.setVisible(false);
          // accept logging -> preceede by starting-up the worklist form
          acceptLogIn(username, password);
        }
        else if (response instanceof EngineLogInReject) { // logging in is REJECTED
          // display the message about the reason of rejection
          MessagePane.inform(logInFrame,
                             ( (EngineLogInReject) response).getReason()
                             /*handler.handle()*/);
        }
      }
    }
    catch (Exception ex) {
      ErrorMessage.showError(logInFrame, ex, "Could not connect to engine.");
    }
  }

  protected int assignmentCount() {
    return this.coordinators.size();
  }

  /**
   *
   * @param username String
   * @param password String
   */
  private void acceptLogIn(String username, String password) {
    // create a fake user object for the username and password
    user = new User(0);
    user.setUserName(username);
    user.setPassword(password);

    // wait for messages from the proxy engine
    this.proxy.start();
    this.begin();
  }


  /**
   * cancelLogIn
   */
  public void cancelLogIn() {
    System.exit(0);
  }

  public void windowOpened(WindowEvent e) {}

  public void windowClosing(WindowEvent e) {
  }

  public void windowClosed(WindowEvent e) {
    proxy.finish();
    this.logOut();
    System.exit(0);
  }

  public void windowIconified(WindowEvent e) {}

  public void windowDeiconified(WindowEvent e) {}

  public void windowActivated(WindowEvent e) {}

  public void windowDeactivated(WindowEvent e) {}


  public void received(String message) {
    IMessage received = null;
    synchronized (this.readLock) {
      received = MessageFactory.getEngineResponse(message);
    }
    if (received != null) {
      this.receive(received);
    }
  }

  private void addAssignment(WorklistAssignmentCoordinator coordinator) {
    coordinator.setListener(this);
    this.added(coordinator);
  }


  /**
   *
   * @param assignmentId String
   * @return AssignmentCoordinator
   */
  public WorklistAssignmentCoordinator getAssignmentCoordinator(Assignment
      assignment) {
    WorklistAssignmentCoordinator coordinator = assignmentCoordinator(
        assignment.getId());
    if (coordinator == null) {
      coordinator = new WorklistAssignmentCoordinator(assignment, this.frame,
              this.getUser());
      coordinators.add(coordinator);
    }
    return coordinator;
  }

  /**
   *
   * @param assignmentId String
   * @return AssignmentCoordinator
   */
  public WorklistAssignmentCoordinator assignmentCoordinator(String
      assignmentId) {
    WorklistAssignmentCoordinator coordinator = null;
    try {
      int id = Integer.parseInt(assignmentId);
      coordinator = assignmentCoordinator(id);
    }
    catch (Exception e) {
      // ignore
    }
    return coordinator;
  }

  /**
   *
   * @param assignmentId String
   * @return AssignmentCoordinator
   */
  public WorklistAssignmentCoordinator assignmentCoordinator(int assignmentId) {
    boolean found = false;
    int i = 0;
    WorklistAssignmentCoordinator coordinator = null;
    while (i < this.coordinators.size() && !found) {
      coordinator = coordinators.get(i++);
      found = (coordinator.getAssignment().getId() == assignmentId);
    }
    return found ? coordinator : null;
  }

  /**
   *
   * @param assignmentId String
   * @return AssignmentCoordinator
   */
  public WorklistAssignmentCoordinator assignmentCoordinator(Assignment
      assignment) {
    boolean found = false;
    int i = 0;
    WorklistAssignmentCoordinator coordinator = null;
    while (i < this.coordinators.size() && !found) {
      coordinator = coordinators.get(i++);
      found = (coordinator.getAssignment() == assignment);
    }
    return found ? coordinator : null;
  }

  private synchronized void request(IMessage message) {
    proxy.request(message);
  }

  /**
   *
   * @param object Object
   */
  public void closeAssignment(Object object) {
    if (object != null) {
      if (object instanceof WorklistAssignmentCoordinator) {
        this.closeAssignment( (WorklistAssignmentCoordinator) object);
      }
    }
  }

  /**
   *
   * @param object Object
   */
  public void closeAssignment(WorklistAssignmentCoordinator coordinator) {
    if (coordinator != null) {
      if (close(coordinator)) {
        if (coordinator.warnClosing()) {
          request(MessageFactory.
                  messageCloseAssignment(coordinator.getAssignment()));
        }
      }
    }
  }

  /**
   *
   * @param id String
   */
  private void assignmentClosed(WorklistAssignmentCoordinator coordinator) {
    if (coordinator != null) {
      this.coordinators.remove(coordinator);
      this.closed(coordinator);
    }
  }


  /**
   *
   * @param coordinator AssignmentCoordinator
   */
  private synchronized void stateChanged(WorklistAssignmentCoordinator
                                         coordinator) {
    if (coordinator != null) {
      coordinator.stateChanged();
      changed(coordinator);
    }
  }

  public IMessage executeRequest(IMessage request) {
    return proxy.request(request);
  }



  /**
   *
   * @param coordinator AssignmentCoordinator
   */
  private synchronized void processRecommendation(WorklistAssignmentCoordinator
                                           coordinator,
                                           Response result) {
    if (coordinator != null) {
      coordinator.processRecommendation(result);
    }
  }


  protected void logOut() {
    IMessage msg = MessageFactory.logOut(Integer.toString(id), user.getUserName(),
                                         user.getPassword());
    request(msg);
  }

  protected void receive(IMessage msg) {
    if (msg instanceof AssignmentMessage) { // AssignmentMessage
      WorklistAssignmentCoordinator coordinator = this.assignmentCoordinator( ( (
          AssignmentMessage) msg).id());
      if (msg instanceof OpenAssignmentMessage) { // OpenAssignment
        openModel(((OpenAssignmentMessage) msg).id(),((OpenAssignmentMessage) msg).getModel());
      }
      else if (msg instanceof CloseAssignmentEngineMessage) { // CloseAssignment
        assignmentClosed(coordinator);
      }
      else if (msg instanceof StateAssignmentMessage) { // StateAssignment
        state(coordinator, ( (StateAssignmentMessage) msg).getState());
      }
      else if (msg instanceof ChangeAssignmentMessage) { // ChangeAssignment
        change(coordinator, ( (ChangeAssignmentMessage) msg).getModel());
      }
      else if (msg instanceof RecommendationAssignmentMessage) { // RecommendationAssignment
        recommendation(coordinator,((RecommendationAssignmentMessage) msg).getRecommendation());
      }
      else if (msg instanceof HistoryAssignmentMessage) { // HistoryAssignment
        history(coordinator,((HistoryAssignmentMessage) msg).getHistory());
      }
      else if (msg instanceof EngineCloseAssignmentAccept) { // CloseAssignmentAccept
        assignmentClosed(coordinator);
      }
      else if (msg instanceof EngineCloseAssignmentReject) { // CloseAssignmentReject
    	  MessagePane.inform(this.frame,( (EngineCloseAssignmentReject) msg).getReason());
      }
    }
  }

  private void openModel(String id, String assignmentString) {
    XMLAssignmentViewBroker broker = (XMLAssignmentViewBroker) XMLBrokerFactory.
        newAssignmentBroker();
    AssignmentModel model = broker.readAssignmentfromString(assignmentString);
    Assignment assignment = new Assignment(new Integer(id).intValue(), model);
    AssignmentState state = new AssignmentState(assignment);
    AssignmentView view = new AssignmentView(state);
    WorklistAssignmentCoordinator coordinator = this.getAssignmentCoordinator(
        assignment);
    coordinator.setView(view);
    broker.readAssignmentGraphicalFromString(model, view, assignmentString);
    this.addAssignment(coordinator);
  }

  private void state(WorklistAssignmentCoordinator coordinator, String state) {
    XMLAssignmnetStateBroker broker = new XMLAssignmnetStateBroker(
        coordinator.getAssignmentState());
    broker.readDocumentString(state);
    broker.fromElement();
    this.stateChanged(coordinator);
  }

  private void change(WorklistAssignmentCoordinator coordinator,
                      String assignmentString) {
    int id = coordinator.getAssignment().getId();

    XMLAssignmentViewBroker broker = (XMLAssignmentViewBroker) XMLBrokerFactory.
        newAssignmentBroker();
    AssignmentModel model = broker.readAssignmentfromString(assignmentString);

    Assignment assignment = new Assignment(id, model);
    AssignmentState state = new AssignmentState(assignment);

    AssignmentView view = new AssignmentView(state);

    coordinator = this.getAssignmentCoordinator(assignment);
    coordinator.setView(view);
    broker.readAssignmentGraphicalFromString(model, view, assignmentString);
    coordinator.change(assignment, view);
  }

  private void recommendation(WorklistAssignmentCoordinator coordinator,
                              String recommendation) {
    try {
      Response result = converter.responseFromXML(recommendation);
      this.processRecommendation(coordinator, result);
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  private void history(WorklistAssignmentCoordinator coordinator,
                       String history) {
    /* HistoryCoordinator history = this.coordinator.getHistoryCoordinator();
         history.update(this.history);*/
  }

public void connected(RemoteEngineProxy engine) {
	// TODO Auto-generated method stub
	
}

public void disconnected(RemoteEngineProxy engine) {
	// TODO Auto-generated method stub
	
}

public void sent(RemoteEngineProxy engine, String msg) {
	// TODO Auto-generated method stub
	
} 
}
