package nl.tue.declare.execution.automaton;


import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import ltl2aut.automaton.*;
 
/**
 * <p>
 * Title: DECLARE
 * </p>
 * 
 * <p>
 * Description:
 * </p>
 * 
 * <p>
 * Copyright: Copyright (c) 2006
 * </p>
 * 
 * <p>
 * Company: TU/e
 * </p>
 * 
 * @author Maja Pesic
 * @version 1.0
 */
public class ExecutableAutomaton {

	private ltl2aut.automaton.Automaton graph;

	private PossibleNodes current;

	public ExecutableAutomaton(ltl2aut.automaton.Automaton graph) {
		super();
		this.graph = graph;
		current = new PossibleNodes();
		current.add(graph.getInit());
	}


	public void ini() {
		current = new PossibleNodes();
		current.add(graph.getInit());
	}

	public PossibleNodes next(String label) {
		if (current != null) {
			current = current.next(label);
		}
		return current;
	}

	/**
	 * 
	 * @param trace
	 *            Collection
	 * @return PossibleStates
	 */
	/*public PossibleStates next(Collection<String> trace) {
		if (trace != null) {
			Iterator<String> iterator = trace.iterator();
			while (iterator.hasNext()) {
				String word = iterator.next();
				this.next(word);
			}
		}
		return current;
	}*/

	/**
	 * 
	 * @param events
	 *            EventLog
	 * @param automaton
	 *            Automaton
	 * @return boolean
	 */
	/*public PossibleNodes executeTrace(String[] word) {
		boolean ok = true;
		if (word != null) {
			ini(); // added now for TEST
			for (int i = 0; (i < word.length && ok); i++) {
				ok = (next(word[i]) != null);
			}
		}
		return current;
	}*/

	public int stateCount() {
		return this.graph.getStateCount();
	}

	public Iterable<State> states() {
		return graph.getStates();
	}

	public PossibleNodes currentState() {
		return this.current;
	}

	public boolean parses(String label) {
		boolean parses = false;
		if (current != null) {
			parses = current.parses(label);
		}
		return parses;
	}

	/**
	 * 
	 * @param trace
	 *            Collection
	 * @return boolean
	 */
	/*public boolean parsesTrace(Collection<String> trace) {
		boolean parses = true;
		if (trace != null) {
			Iterator<String> iterator = trace.iterator();
			while (iterator.hasNext() && parses) {
				String word = iterator.next();
				parses = this.parses(word);
			}
		}
		return parses;
	}*/

	/**
	 * 
	 * @param label
	 *            String
	 * @return boolean
	 */
	public boolean reachableTransition(String label) {
		boolean reachable = false;
		if (current != null) {
			reachable = this.current.reachableTransition(new ArrayList<State>(),
					label);
		}
		return reachable;
	}

	/**
	 * isEmpty
	 */
	public boolean isEmpty() {
		return graph.getStateCount() == 0;
	}

	/**
	 * Get all states that have an input transition that parses the given label.
	 * 
	 * @param label
	 *            String
	 * @return Iterator
	 */
	private ArrayList<State> input(final String label) {
		final ArrayList<State> result = new ArrayList<State>();
		for (State s: graph.getStates()){
			if (AutomatonUtils.hasInput(s, label)) {
				result.add(s);
			}			
		}
		// gather all reachable states
		/*graph.forAllNodes(new Visitor() {

			public void visitEdge(Transition arg0) {
				//ignore
			}

			public void visitNode(Node node) {
				if (AutomatonUtils.hasInput(node, label)) {
					result.add(node);
				}
			}
		});*/
		return result;
	}

    /*
     * Checks if the automaton accepts a word word where two literals (first and second) appear in one after another. 
     * The wanted form of the word is *first*second*.
     */
	public boolean order(String first, String second) {
		Iterator<State> firstStatesIterator = this.input(first).iterator();
		Collection<State> secondStates = this.input(second);
		boolean found = false;
		while (firstStatesIterator.hasNext() && !found) {
			State stateFirst = firstStatesIterator.next();
			ArrayList<State> reachable = new ArrayList<State>();
			AutomatonUtils.reachableStates(stateFirst, reachable);
			Iterator<State> secondStatesIterator = secondStates.iterator();
			while (secondStatesIterator.hasNext() && !found) {
				State stateSecond = secondStatesIterator.next();
				found = reachable.contains(stateSecond);
			}
		}
		return found;
	}
}
