package nl.tue.declare.execution.msg;

import java.util.StringTokenizer;

import nl.tue.declare.domain.instance.Assignment;
import nl.tue.declare.domain.instance.Event;
import nl.tue.declare.execution.AssignmentState;
import nl.tue.declare.execution.Execution;
import nl.tue.declare.execution.WorkItem;
import nl.tue.declare.execution.msg.engine.msg.*;
import nl.tue.declare.execution.msg.engine.response.*;
import nl.tue.declare.execution.msg.worklist.*;

import org.processmining.models.operationalsupport.Response;

import yawlservice.ExternalCase;

/**
 * <p>Title: DECLARE</p>
 *
 * <p>Description: </p>
 *
 * <p>Copyright: Copyright (c) 2006</p>
 *
 * <p>Company: TU/e</p>
 *
 * @author Maja Pesic
 * @version 1.0
 */
public class MessageFactory {

  public static IMessage getWorklistRequest(String msg) {
    if (msg == null) {
      return new UnknownMessage();
    }
    StringTokenizer st = new StringTokenizer(msg, IMessage.SEPARATOR);

    // the first token in every message is CLASS
    String CLASS = st.nextToken();

    if (CLASS.equals(UserMessage.CLASS)) { // CLASS == USER
      // the second token in a UserMessage is TYPE
      String TYPE = st.nextToken();
      String id = st.nextToken();
      String userName = st.nextToken();
      String password = st.nextToken();
      if (TYPE.equals(MessageLogIn.TYPE)) { // log in
        return new MessageLogIn(id, userName, password);
      }
      else if (TYPE.equals(MessageLogOut.TYPE)) { // log out
        return new MessageLogOut(id, userName, password);
      }
    }
    else if (CLASS.equals(AssignmentMessage.CLASS)) { // CLASS == ASSIGNMENT
      // the second token in a AssignmentMessage is TYPE
      String TYPE = st.nextToken();
      // the third token in a AssignmentMessage is assignmentId
      String assignmentId = st.nextToken();
      if (TYPE.equals(MessageCloseAssignment.TYPE)) { // close assignment
        return new MessageCloseAssignment(assignmentId);
      }
      else if (TYPE.equals(WorkItemMessage.TYPE)) { // TYPE == WORKITEM
        String itemId = st.nextToken(); // work item id
        String activity = st.nextToken(); // take the activity ID
        String event = st.nextToken(); // take the event type
        String ACTION = st.nextToken();
        String xml = st.nextToken(); // take xml
        if (ACTION.equals(RequestEventMessage.ACTION)) {
          return new RequestEventMessage(assignmentId, activity, event, itemId,
                                         xml);
        } else {
          if (ACTION.equals(ForwardEventMessage.ACTION)) {
            String ec = st.nextToken();
                    return new ForwardEventMessage(assignmentId, activity, event, itemId,
                                                   xml,ec);
        }
        }
      }
    }
    return new UnknownMessage();
  }

  /**
   *
   * @param msg String
   * @return IMessage
   */
  public static IMessage getEngineResponse(String msg) {
    if (msg == null) {
      return new UnknownMessage();
    }
    StringTokenizer st = new StringTokenizer(msg, IMessage.SEPARATOR);

    // the first token in every message is CLASS
    String CLASS = st.nextToken();

    if (CLASS.equals(UserMessage.CLASS)) { // CLASS == UserMessage
      // the second token in every message is TYPE
      String TYPE = st.nextToken();
      if (TYPE.equals(EngineLogInAccept.TYPE)) { // TYPE == accepted
        String id = st.nextToken();
        return logInAccept(id);
      }
      else if (TYPE.equals(EngineLogInReject.TYPE)) { // TYPE == rejected
        String description = st.nextToken();
        return logInReject(description);
        // *** handler = new WorklistMessageHandlerLogInReject(description);
      }
    }
    else if (CLASS.equals(AssignmentMessage.CLASS)) { // // CLASS == AssignmentMessage
      // the second token in every message is TYPE
      String TYPE = st.nextToken();
      String caseId = st.nextToken();
      if (TYPE.equals(OpenAssignmentMessage.TYPE)) { // TYPE == OpenAssignment
        String model = st.nextToken();
        // *** handler = new OpenAssignmentWorklistHandler(caseId, model);
        return new OpenAssignmentMessage(caseId, model);
      }
      else if (TYPE.equals(CloseAssignmentEngineMessage.TYPE)) { // TYPE == CloseAssignment
        // *** handler = new CloseAssignmentWorklistHandlerAccept(caseId);
        return new CloseAssignmentEngineMessage(caseId);
      }
      else if (TYPE.equals(StateAssignmentMessage.TYPE)) { // TYPE == StateAssignment
        String state = st.nextToken();
        return new StateAssignmentMessage(caseId, state);
        // *** handler = new StateAssignmentWorklistHandler(caseId, state);
      }
      else if (TYPE.equals(ChangeAssignmentMessage.TYPE)) { // TYPE == ChangeAssignment
        String state = st.nextToken();
        return new ChangeAssignmentMessage(caseId, state);
        // *** handler = new ChangeAssignmentWorklistHandler(caseId, state);
      }
      else if (TYPE.equals(RecommendationAssignmentMessage.TYPE)) { // TYPE == RecommendationAssignment
        String recommendation = st.nextToken();
        return new RecommendationAssignmentMessage(caseId, recommendation);
        // *** handler = new RecommendationAssignmentWorklistHandler(caseId, recommendation);
      }
      else if (TYPE.equals(HistoryAssignmentMessage.TYPE)) { // TYPE == HistoryAssignment
        String history = st.nextToken();
        return new HistoryAssignmentMessage(caseId, history);
        // *** handler = new HistoryAssignmentWorklistHandler(caseId, history);
      }
      else if (TYPE.equals(WorkItemMessage.TYPE)) { // TYPE = WorkItem EventAccept
        String wiId = st.nextToken();
        String actId = st.nextToken();
        String type = st.nextToken();
        String ACTION = st.nextToken();
        String xml = st.nextToken();
        if (ACTION.equals(EngineEventAccept.ACTION)){
          return new EngineEventAccept(caseId, actId, type, wiId, xml);
        } else if (TYPE.equals(EngineEventReject.TYPE)) { // TYPE = EventReject
          String reason = st.hasMoreTokens()?st.nextToken():"";
          return new EngineEventReject(caseId, wiId, actId, type, xml, reason);
        }
      }
      else if (TYPE.equals(EngineCloseAssignmentAccept.TYPE)) { // TYPE == CloseAssignmentAccept
        return new EngineCloseAssignmentAccept(caseId);
      }
      else if (TYPE.equals(EngineCloseAssignmentReject.TYPE)) { // TYPE == CloseAssignmentAccept
        String info = st.nextToken();
        return new EngineCloseAssignmentReject(caseId, info);
      }
    }
    return new UnknownMessage();
  }

  /**
   *
   * @param state AssignmentState
   * @return StateAssignmentMessage
   */
  public static IMessage createStateAssignmentMassage( //Assignment assignment,
      AssignmentState state) {
    Assignment assignment = state.getAssignment();
    return new StateAssignmentMessage(assignment, state);
  }

  public static IMessage createHistoryAssignmentMassage(Execution execution) {
    return new HistoryAssignmentMessage(execution.getAssignment(), execution);
  }

  /**
   *
   * @param assignment Assignment
   * @param model String
   * @return OpenAssignmentMessage
   */
  public static IMessage createOpenAssignementMessage(Assignment
      assignment, String model) {
    return new OpenAssignmentMessage(assignment, model);
  }

  /**
   *
   * @param assignment AssignmentState
   * @param model String
   * @return StateAssignmentMessage
   */
  public static IMessage createChangeAssignmentMassage(Assignment assignment,
      String model) {
    return new ChangeAssignmentMessage(assignment, model);
  }

  public static IMessage messageCloseAssignment(Assignment assignemnt) {
    return new MessageCloseAssignment(assignemnt);
  }

  public static IMessage eventRequest(WorkItem wi, Event event) {
    return new RequestEventMessage(wi,event);
  }

  public static IMessage eventForward(WorkItem wi, Event event, ExternalCase ec) {
    return new ForwardEventMessage(wi,event,ec);
  }

  public static IMessage messageRecommendationAssignment(
      Assignment assignment, Response recommendationResult) {
    return new RecommendationAssignmentMessage(assignment, recommendationResult);
  }

  public static IMessage messageCloseAssignmentEngineMessage(Assignment
      assignment) {
    return new CloseAssignmentEngineMessage(assignment);
  }

  public static IMessage logOut(String id, String user, String pass) {
    return new MessageLogOut(id, user, pass);
  }

  public static IMessage logInAccept(String id) {
    return new EngineLogInAccept(id);
  }

  public static IMessage logInReject(String reason) {
    return new EngineLogInReject(reason);
  }
}
