package nl.tue.declare.utils;

import java.io.*;
import javax.xml.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;
import javax.xml.validation.*;

import org.w3c.dom.*;
import org.xml.sax.*;

/**
 *
 * <p>Title: DECLARE</p>
 *
 * <p>Description: </p>
 *
 * <p>Copyright: Copyright (c) 2006</p>
 *
 * <p>Company: TU/e</p>
 *
 * @author Maja Pesic
 * @version 1.0
 */
public class XMLParser {
  public static Element getFirstElement(Element element,
                                        String name) {
    NodeList nl = element.getElementsByTagName(name);
    if (nl.getLength() < 1) {
      return null;
    }
    return (Element) nl.item(0);
  }

  /**
   *
   * @param node Element
   * @param name String
   * @return String
   */
  public static String getSimpleElementText(Element node, String name) {
    Element nameEl = XMLParser.getFirstElement(node, name);
    Node textNode = nameEl.getFirstChild();
    if (textNode instanceof Text) {
      return textNode.getNodeValue();
    }
    else {
      return null;
    }
  }

  /**
   *
   * @param node Node
   * @return String
   */
  public static String toString(Node node) {
    try {
      StringWriter stringWriter = new StringWriter();
      Source source = new DOMSource(node);
      Result result = new StreamResult(stringWriter);

      transform(source, result);
      return stringWriter.toString();
    }
    catch (Exception ex) {
      ex.printStackTrace();
      return "blank";
    }
  }

  /**
   *
   * @param node Node
   * @param file String
   */
  public static void toFile(Node node, String file) {
    try {
      File outputFile = new File(file);
      OutputStream ostream = new FileOutputStream(outputFile);

      Source source = new DOMSource(node);
      Result result = new StreamResult(ostream);

      transform(source, result);
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  /**
   *
   * @param XML String
   * @return Node
   */
  public static Node fromString(String XML) {
    try {
      Source source = new StreamSource(new StringReader(XML));
      DOMResult result = new DOMResult();

      transform(source, result);
      return result.getNode();
    }
    catch (Exception e) {
      e.printStackTrace();
      return null;
    }
  }

  /**
   *
   * @param source Source
   * @param result Result
   */
  private static void transform(Source source, Result result) {
    TransformerFactory tf = TransformerFactory.newInstance();
    Transformer transformer = null;
    try {
      transformer = tf.newTransformer();
      transformer.transform(source, result);
    }
    catch (TransformerConfigurationException ex) {
      ex.printStackTrace();
    }
    catch (TransformerException ex1) {
      ex1.printStackTrace();
    }
  }

  /**
   *
   * @param node Node
   */
  public static void print(Node node) {
    SystemOutWriter.singleton().println("HERE " + XMLParser.toString(node));
  }

  /**
   *
   * @param schemaSource Source
   * @param xml Source
   * @return boolean
   */
  public static boolean validate(Source schemaSource, Source xml) {
    boolean valid = false;
    SchemaFactory factory = SchemaFactory.newInstance(XMLConstants.
        W3C_XML_SCHEMA_NS_URI);
    try {
      Schema schema = factory.newSchema(schemaSource);
      Validator validator = schema.newValidator();

      try {
        validator.validate(xml);
        valid = true;
      }
      catch (IOException ex1) {
        ex1.printStackTrace();
      }
      catch (SAXException ex1) {
        ex1.printStackTrace();
      }
    }
    catch (SAXException ex) {
      ex.printStackTrace();
    }
    return valid;
  }

  /**
   *
   * @param schema String
   * @param xml String
   * @return boolean
   */
  public static boolean validate(String schema, String xml) {
    Source schemaSource = new StreamSource(new StringReader(schema));
    Source xmlSource = new StreamSource(new StringReader(xml));
    return validate(schemaSource, xmlSource);
  }
}
